import { computed, type Ref, ref } from 'vue'
import type { EChartsOption } from 'echarts/types/dist/shared'
import axiosInstance from '@/utils/axios/axiosInstance.ts'
import type { ResponseInfo } from '@/types/res.ts'
import type { QueryInfo } from '@/types/table.ts'
import { generateUniqueColors } from '@/utils/common'

interface UserBehaviorChartItem {
  name: string
  value: number
}

interface ChartInfo {
  PieChart: UserBehaviorChartItem[]
  BarChart: {
    labelList: string[]
    valueList: number[]
  }
}

export function useUserBehaviorChart(
  url: string,
  queryFormData: Ref<any>,
  filterInfo: Array<QueryInfo>,
  chartNeedFields: Array<string>,
  isPie: Ref<boolean>
) {
  const chartInfo = ref<ChartInfo>({
    PieChart: [],
    BarChart: {
      labelList: [],
      valueList: []
    }
  })

  const updateChartData = async () => {
    if (chartNeedFields.length > 0) {
      const hasFilter = chartNeedFields.every((item) => {
        return queryFormData.value[item] === ''
      })
      if (hasFilter) {
        const tip = filterInfo
          .filter((item) => {
            return chartNeedFields.includes(item.name as any)
          })
          .map((item) => {
            return item.label
          })
          .join(',')
        ElMessage.warning(`${tip}请至少输入一个筛选条件`)
        return
      }
    }

    const params = {} as any
    Object.assign(params, queryFormData.value)
    if (queryFormData.value.createTime) {
      params.createTime = queryFormData.value.createTime
        .map((item: any) => {
          return new Date(item).getTime().toString()
        })
        .join(',')
    }

    const res = (await axiosInstance.post(url, params)) as ResponseInfo
    if (res.code !== 0) {
      ElMessage.error('获取数据失败')
      return
    }

    const data = res.data as Array<{
      name: string
      count: number
    }> | null

    if (!data) {
      chartInfo.value.PieChart = []
      chartInfo.value.BarChart = {
        labelList: [],
        valueList: []
      }
    } else {
      const isAllZero = data.every((item) => item.count === 0)
      if (isAllZero) {
        chartInfo.value.PieChart = []
        chartInfo.value.BarChart = {
          labelList: [],
          valueList: []
        }
        return
      }

      chartInfo.value.PieChart = data.map((item) => ({
        name: item.name,
        value: item.count
      }))
      chartInfo.value.BarChart = {
        labelList: data.map((item) => item.name),
        valueList: data.map((item) => item.count)
      }
    }
  }

  const pieChartOptions = computed<EChartsOption>(() => {
    const colors = generateUniqueColors(chartInfo.value.PieChart.length)
    return {
      color: colors,
      tooltip: {
        trigger: 'item',
        formatter: (params: any) => {
          const data = chartInfo.value.PieChart[params.dataIndex]
          return `${data.name}<br/>
                  数量: ${data.value}`
        }
      },
      legend: {
        top: '5%',
        left: 'center',
        itemGap: 20,
        textStyle: {
          fontSize: 12
        }
      },
      series: [
        {
          name: '用户行为',
          type: 'pie',
          radius: ['40%', '70%'],
          center: ['50%', '50%'],
          avoidLabelOverlap: false,
          itemStyle: {
            borderRadius: 10,
            borderColor: '#fff',
            borderWidth: 2
          },
          label: {
            show: false,
            position: 'center'
          },
          emphasis: {
            label: {
              show: true,
              fontSize: 20,
              fontWeight: 'bold'
            }
          },
          labelLine: {
            show: false
          },
          data: chartInfo.value.PieChart
        }
      ]
    }
  })

  const barChartOptions = computed<EChartsOption>(() => {
    const barChartInfo = chartInfo.value.BarChart
    const colors = generateUniqueColors(barChartInfo.labelList.length)
    return {
      xAxis: {
        type: 'category',
        data: barChartInfo.labelList,
        axisLabel: {
          interval: 0,
          rotate: 30
        }
      },
      yAxis: {
        type: 'log',
        min: 1
      },
      color: colors,
      tooltip: {
        trigger: 'axis',
        axisPointer: {
          type: 'shadow'
        },
        formatter: (params: any) => {
          const data = chartInfo.value.PieChart[params[0].dataIndex]
          return `${data.name}<br/>
                  数量: ${data.value}`
        }
      },
      grid: {
        top: '15%',
        left: '3%',
        right: '4%',
        bottom: '15%',
        containLabel: true
      },
      series: [
        {
          name: '用户行为',
          data: barChartInfo.valueList,
          type: 'bar',
          barWidth: '10%',
          label: {
            show: true,
            position: 'top',
            formatter: '{c}'
          },
          itemStyle: {
            // color: (params) => {
            //   return colors[params.dataIndex]
            // }
          }
        }
      ]
    }
  })

  const chartOptions = computed(() => {
    const pieChartInfo = chartInfo.value.PieChart.length ?? null
    const barChartInfo = chartInfo.value.BarChart.valueList.length ?? null
    if (!pieChartInfo && !barChartInfo) return null
    return isPie.value ? pieChartOptions.value : barChartOptions.value
  })

  return {
    updateChartData,
    chartOptions
  }
}
